package com.xyzc.start.service.impl;

import com.xyzc.common.core.exception.ErrorCode;
import com.xyzc.common.core.utils.AssertUtils;
import com.xyzc.common.core.utils.Result;
import com.xyzc.common.web.security.user.SecurityUser;
import com.xyzc.start.domain.MemberCard;
import com.xyzc.start.domain.Scheme;
import com.xyzc.start.domain.vo.*;
import com.xyzc.start.mapper.AiModelMapper;
import com.xyzc.start.mapper.EquityMapper;
import com.xyzc.start.mapper.MemberCardMapper;
import com.xyzc.start.mapper.SchemeMapper;
import com.xyzc.start.service.ISchemeService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.xbill.DNS.dnssec.R;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class SchemeServiceImpl implements ISchemeService {

    private final SchemeMapper schemeMapper;
    private final MemberCardMapper memberCardMapper;
    private final EquityMapper equityMapper;
    private final AiModelMapper aiModelMapper;

    /**
     * 获取当前方案
     *
     * @return
     */
    @Override
    public SchemeVo getCurrentMember() {
        SchemeVo scheme = schemeMapper.getCurrentMember();

        // 有且只有一个会员卡套餐生效
        // 根据套餐查询会员卡
        List<MemberCardVo> memberCardVos = memberCardMapper.queryVoListBySchemeId(scheme.getId());
        // 让memberCardVos中根据memberId合并成新的List
        List<List<MemberCardVo>> memberList = new ArrayList<>();
        List<MemberCardVo> collect = new ArrayList<>();
        //TODO ： 添加立省金额
        for (MemberCardVo memberCardVo : memberCardVos) {
            // collect不为空并且memberCardId不一致时新建一个list
            if (!collect.isEmpty() &&
                    !collect.get(0).getMemberCardId().equals(memberCardVo.getMemberCardId())) {
                memberList.add(collect);
                collect = new ArrayList<>();
            }
            // 对当前值进行处理
            memberCardVo.setSaveMoney(memberCardVo.getPrice().subtract(memberCardVo.getStrikethroughPrice()));
            collect.add(memberCardVo);
        }
        memberList.add(collect);

        // 将会员卡列表绑定到方案中
        scheme.setMemberCardList(memberList);

        return scheme;
    }

    /**
     * 根据id获取燃料信息
     *
     * @param fuelId
     * @return
     */
    @Override
    public FuelPackageVo queryCurrentFuelById(Long fuelId) {

        return schemeMapper.queryFuelPackageById(SecurityUser.getUserId(), fuelId);
    }

    /**
     * 获取
     *
     * @param modelId 模型id
     * @return
     */
    @Override
    public List<AiModelLawVo> queryModelLawList(Long modelId) {

        return aiModelMapper.queryModelLawList(modelId);
    }

    /**
     * 根据会员卡id获取会员卡信息
     *
     * @param memberId
     * @return
     */
    @Override
    public MemberCardVo queryCurrentMemberById(Long memberId) {

        // 传入用户自己的值去
        return schemeMapper.queryMemberCardByMemberId(SecurityUser.getUserId(), memberId);
    }

    /**
     * 获取当前生效的燃料方案
     *
     * @return
     */
    @Override
    public SchemeVo getCurrentFuel() {
        List<SchemeVo> currentFuel = schemeMapper.getCurrentFuel();
        // 有且只有一个会员卡套餐生效
        AssertUtils.isUnique(currentFuel, ErrorCode.SCHEME_NOT_UNIQUE);

        return currentFuel.get(0);
    }

}
